@@ -4,7 +4,7 @@
  * Copyright (C) 1999
  *  David Corcoran <corcoran@linuxnet.com>
  *
- * $Id: atrhandler.h 2891 2008-04-21 13:14:46Z rousseau $
+ * $Id: atrhandler.h 3260 2009-01-02 15:19:59Z rousseau $
  */
 
 /**
@@ -50,7 +50,7 @@ extern "C"
 	 * Decodes the ATR and fills the structure
 	 */
 
-	short ATRDecodeAtr(PSMARTCARD_EXTENSION psExtension,
+	short ATRDecodeAtr(/*@out@*/ PSMARTCARD_EXTENSION psExtension,
 		PUCHAR pucAtr, DWORD dwLength);
 
 #ifdef __cplusplus
