@@ -6,7 +6,7 @@
  * Copyright (C) 2004
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: eventhandler.h 2639 2007-10-11 08:37:33Z rousseau $
+ * $Id: eventhandler.h 3259 2009-01-02 14:32:44Z rousseau $
  */
 
 /**
@@ -43,7 +43,8 @@ extern "C"
 	READER_STATE, *PREADER_STATE;
 
 	LONG EHInitializeEventStructures(void);
-	LONG EHSpawnEventHandler(PREADER_CONTEXT);
+	LONG EHSpawnEventHandler(PREADER_CONTEXT,
+		/*@null@*/ RESPONSECODE (*)(DWORD));
 	LONG EHDestroyEventHandler(PREADER_CONTEXT);
 
 #ifdef __cplusplus
