@@ -4,7 +4,7 @@
  * Copyright (C) 2000-2003
  *  David Corcoran <corcoran@linuxnet.com>
  *
- * $Id: hotplug_generic.c 1786 2006-01-03 17:11:29Z rousseau $
+ * $Id: hotplug_generic.c 2748 2008-01-21 09:10:58Z rousseau $
  */
 
 /**
@@ -25,7 +25,7 @@
 #define FALSE 0
 #endif
 
-#if !defined(__APPLE__) && !defined(HAVE_LIBUSB) && !defined(__linux__)
+#if !defined(__APPLE__) && !defined(HAVE_LIBUSB) && !defined(__linux__) && !defined(HAVE_LIBHAL)
 
 char ReCheckSerialReaders = FALSE;
 
