@@ -7,7 +7,7 @@
  *  Toni Andjelkovic <toni@soth.at>
  *  Damien Sauveron <damien.sauveron@labri.fr>
  *
- * $Id: hotplug_libusb.c 2918 2008-04-30 07:33:58Z rousseau $
+ * $Id: hotplug_libusb.c 3197 2008-11-10 09:18:27Z rousseau $
  */
 
 /**
@@ -38,6 +38,7 @@
 #include "winscard_msg.h"
 #include "sys_generic.h"
 #include "hotplug.h"
+#include "utils.h"
 
 #undef DEBUG_HOTPLUG
 #define ADD_SERIAL_NUMBER
@@ -506,15 +507,25 @@ static LONG HPAddHotPluggable(struct usb_device *dev, const char bus_device[],
 		usb_dev_handle *device;
 		char serialNumber[MAX_READERNAME];
 		char fullname[MAX_READERNAME];
+		int ret;
 
 		device = usb_open(dev);
-		usb_get_string_simple(device, dev->descriptor.iSerialNumber,
+		ret = usb_get_string_simple(device, dev->descriptor.iSerialNumber,
 			serialNumber, MAX_READERNAME);
 		usb_close(device);
 
-		snprintf(fullname, sizeof(fullname), "%s (%s)",
-			driver->readerName, serialNumber);
-		readerTracker[i].fullName = strdup(fullname);
+		if (ret < 0)
+		{
+			Log2(PCSC_LOG_ERROR, "usb_get_string_simple failed: %s",
+				usb_strerror());
+			readerTracker[i].fullName = strdup(driver->readerName);
+		}
+		else
+		{
+			snprintf(fullname, sizeof(fullname), "%s (%s)",
+				driver->readerName, serialNumber);
+			readerTracker[i].fullName = strdup(fullname);
+		}
 	}
 	else
 #endif
@@ -524,8 +535,12 @@ static LONG HPAddHotPluggable(struct usb_device *dev, const char bus_device[],
 		driver->libraryPath, deviceName) == SCARD_S_SUCCESS)
 		readerTracker[i].status = READER_PRESENT;
 	else
+	{
 		readerTracker[i].status = READER_FAILED;
 
+		(void)CheckForOpenCT();
+	}
+
 	SYS_MutexUnLock(&usbNotifierMutex);
 
 	return 1;
