@@ -6,7 +6,7 @@
  *  Damien Sauveron <damien.sauveron@labri.fr>
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: ifdwrapper.c 2944 2008-05-12 08:36:21Z rousseau $
+ * $Id: ifdwrapper.c 3294 2009-01-28 13:34:28Z rousseau $
  */
 
 /**
@@ -65,9 +65,11 @@ LONG IFDSetPTS(PREADER_CONTEXT rContext, DWORD dwProtocol, UCHAR ucFlags,
 #endif
 
 	/*
-	 * LOCK THIS CODE REGION
+	 * Locking is done in winscard.c SCardConnect() and SCardReconnect()
+	 *
+	 * This avoids to renegociate the protocol and confuse the card
+	 * Error returned by CCID driver is: CCID_Receive Procedure byte conflict
 	 */
-	SYS_MutexLock(rContext->mMutex);
 
 	ucValue[0] = rContext->dwSlot;
 
@@ -75,7 +77,7 @@ LONG IFDSetPTS(PREADER_CONTEXT rContext, DWORD dwProtocol, UCHAR ucFlags,
 	if (rContext->dwVersion == IFD_HVERSION_1_0)
 	{
 	        ucValue[0] = rContext->dwSlot;
-	        IFDSetCapabilities(rContext, TAG_IFD_SLOTNUM, 1, ucValue);
+	        (void)IFDSetCapabilities(rContext, TAG_IFD_SLOTNUM, 1, ucValue);
 	        rv = (*IFD_set_protocol_parameters) (dwProtocol,
 			ucFlags, ucPTS1, ucPTS2, ucPTS3);
 	}
@@ -90,7 +92,7 @@ LONG IFDSetPTS(PREADER_CONTEXT rContext, DWORD dwProtocol, UCHAR ucFlags,
 	if (rContext->dwVersion == IFD_HVERSION_1_0)
 	{
 	        ucValue[0] = rContext->dwSlot;
-	        IFDSetCapabilities(rContext, TAG_IFD_SLOTNUM, 1, ucValue);
+	        (void)IFDSetCapabilities(rContext, TAG_IFD_SLOTNUM, 1, ucValue);
 		rv = IFD_Set_Protocol_Parameters(dwProtocol, ucFlags, ucPTS1,
 			ucPTS2, ucPTS3);
 	}
@@ -101,11 +103,6 @@ LONG IFDSetPTS(PREADER_CONTEXT rContext, DWORD dwProtocol, UCHAR ucFlags,
 	}
 #endif
 
-	SYS_MutexUnLock(rContext->mMutex);
-	/*
-	 * END OF LOCKED REGION
-	 */
-
 	return rv;
 }
 
@@ -137,11 +134,9 @@ LONG IFDOpenIFD(PREADER_CONTEXT rContext)
 		}
 #endif
 
-	/*
-	 * LOCK THIS CODE REGION
-	 */
+	/* LOCK THIS CODE REGION */
+	(void)SYS_MutexLock(rContext->mMutex);
 
-	SYS_MutexLock(rContext->mMutex);
 #ifndef PCSCLITE_STATIC_DRIVER
 	if (rContext->dwVersion == IFD_HVERSION_1_0)
 	{
@@ -173,11 +168,9 @@ LONG IFDOpenIFD(PREADER_CONTEXT rContext)
 			rv = IFDHCreateChannel(rContext->dwSlot, rContext->dwPort);
 	}
 #endif
-	SYS_MutexUnLock(rContext->mMutex);
 
-	/*
-	 * END OF LOCKED REGION
-	 */
+	/* END OF LOCKED REGION */
+	(void)SYS_MutexUnLock(rContext->mMutex);
 
 	return rv;
 }
@@ -188,6 +181,7 @@ LONG IFDOpenIFD(PREADER_CONTEXT rContext)
 LONG IFDCloseIFD(PREADER_CONTEXT rContext)
 {
 	RESPONSECODE rv = IFD_SUCCESS;
+	int repeat;
 
 #ifndef PCSCLITE_STATIC_DRIVER
 	RESPONSECODE(*IO_close_channel) (void) = NULL;
@@ -199,13 +193,21 @@ LONG IFDCloseIFD(PREADER_CONTEXT rContext)
 		IFDH_close_channel = rContext->psFunctions.psFunctions_v2.pvfCloseChannel;
 #endif
 
-	/*
-	 * LOCK THIS CODE REGION
-	 */
-
+	/* TRY TO LOCK THIS CODE REGION */
+	repeat = 5;
+again:
 	rv = SYS_MutexTryLock(rContext->mMutex);
 	if (EBUSY == rv)
+	{
 		Log1(PCSC_LOG_ERROR, "Locking failed");
+		repeat--;
+		if (repeat)
+		{
+			(void)SYS_USleep(100*1000);	/* 100 ms */
+			goto again;
+		}
+	}
+
 #ifndef PCSCLITE_STATIC_DRIVER
 	if (rContext->dwVersion == IFD_HVERSION_1_0)
 
@@ -218,11 +220,9 @@ LONG IFDCloseIFD(PREADER_CONTEXT rContext)
 	else
 		rv = IFDHCloseChannel(rContext->dwSlot);
 #endif
-	SYS_MutexUnLock(rContext->mMutex);
 
-	/*
-	 * END OF LOCKED REGION
-	 */
+	/* END OF LOCKED REGION */
+	(void)SYS_MutexUnLock(rContext->mMutex);
 
 	return rv;
 }
@@ -278,8 +278,8 @@ LONG IFDGetCapabilities(PREADER_CONTEXT rContext, DWORD dwTag,
 	RESPONSECODE rv = IFD_SUCCESS;
 
 #ifndef PCSCLITE_STATIC_DRIVER
-	RESPONSECODE(*IFD_get_capabilities) (DWORD, PUCHAR) = NULL;
-	RESPONSECODE(*IFDH_get_capabilities) (DWORD, DWORD, PDWORD, PUCHAR) = NULL;
+	RESPONSECODE(*IFD_get_capabilities) (DWORD, /*@out@*/ PUCHAR) = NULL;
+	RESPONSECODE(*IFDH_get_capabilities) (DWORD, DWORD, PDWORD, /*@out@*/ PUCHAR) = NULL;
 
 	if (rContext->dwVersion == IFD_HVERSION_1_0)
 		IFD_get_capabilities =
@@ -289,11 +289,8 @@ LONG IFDGetCapabilities(PREADER_CONTEXT rContext, DWORD dwTag,
 			rContext->psFunctions.psFunctions_v2.pvfGetCapabilities;
 #endif
 
-	/*
-	 * LOCK THIS CODE REGION
-	 */
-
-	SYS_MutexLock(rContext->mMutex);
+	/* LOCK THIS CODE REGION */
+	(void)SYS_MutexLock(rContext->mMutex);
 
 #ifndef PCSCLITE_STATIC_DRIVER
 	if (rContext->dwVersion == IFD_HVERSION_1_0)
@@ -309,11 +306,8 @@ LONG IFDGetCapabilities(PREADER_CONTEXT rContext, DWORD dwTag,
 			pucValue);
 #endif
 
-	SYS_MutexUnLock(rContext->mMutex);
-
-	/*
-	 * END OF LOCKED REGION
-	 */
+	/* END OF LOCKED REGION */
+	(void)SYS_MutexUnLock(rContext->mMutex);
 
 	return rv;
 }
@@ -345,7 +339,7 @@ LONG IFDPowerICC(PREADER_CONTEXT rContext, DWORD dwAction,
 	/*
 	 * Check that the card is inserted first
 	 */
-	IFDStatusICC(rContext, &dwStatus, pucAtr, pdwAtrLen);
+	(void)IFDStatusICC(rContext, &dwStatus, pucAtr, pdwAtrLen);
 
 	if (dwStatus & SCARD_ABSENT)
 		return SCARD_W_REMOVED_CARD;
@@ -356,17 +350,14 @@ LONG IFDPowerICC(PREADER_CONTEXT rContext, DWORD dwAction,
 		IFDH_power_icc = rContext->psFunctions.psFunctions_v2.pvfPowerICC;
 #endif
 
-	/*
-	 * LOCK THIS CODE REGION
-	 */
-
-	SYS_MutexLock(rContext->mMutex);
+	/* LOCK THIS CODE REGION */
+	(void)SYS_MutexLock(rContext->mMutex);
 
 #ifndef PCSCLITE_STATIC_DRIVER
 	if (rContext->dwVersion == IFD_HVERSION_1_0)
 	{
 		ucValue[0] = rContext->dwSlot;
-		IFDSetCapabilities(rContext, TAG_IFD_SLOTNUM, 1, ucValue);
+		(void)IFDSetCapabilities(rContext, TAG_IFD_SLOTNUM, 1, ucValue);
 		rv = (*IFD_power_icc) (dwAction);
 	}
 	else
@@ -380,17 +371,15 @@ LONG IFDPowerICC(PREADER_CONTEXT rContext, DWORD dwAction,
 	if (rContext->dwVersion == IFD_HVERSION_1_0)
 	{
 		ucValue[0] = rContext->dwSlot;
-		IFDSetCapabilities(rContext, TAG_IFD_SLOTNUM, 1, ucValue);
+		(void)IFDSetCapabilities(rContext, TAG_IFD_SLOTNUM, 1, ucValue);
 		rv = IFD_Power_ICC(dwAction);
 	}
 	else
 		rv = IFDHPowerICC(rContext->dwSlot, dwAction, pucAtr, pdwAtrLen);
 #endif
-	SYS_MutexUnLock(rContext->mMutex);
 
-	/*
-	 * END OF LOCKED REGION
-	 */
+	/* END OF LOCKED REGION */
+	(void)SYS_MutexUnLock(rContext->mMutex);
 
 	/* use clean values in case of error */
 	if (rv != IFD_SUCCESS)
@@ -400,7 +389,7 @@ LONG IFDPowerICC(PREADER_CONTEXT rContext, DWORD dwAction,
 
 		if (rv == IFD_NO_SUCH_DEVICE)
 		{
-			SendHotplugSignal();
+			(void)SendHotplugSignal();
 			return SCARD_E_READER_UNAVAILABLE;
 		}
 
@@ -411,7 +400,7 @@ LONG IFDPowerICC(PREADER_CONTEXT rContext, DWORD dwAction,
 	 * Get the ATR and it's length
 	 */
 	if (rContext->dwVersion == IFD_HVERSION_1_0)
-		IFDStatusICC(rContext, &dwStatus, pucAtr, pdwAtrLen);
+		(void)IFDStatusICC(rContext, &dwStatus, pucAtr, pdwAtrLen);
 
 	return rv;
 }
@@ -431,7 +420,7 @@ LONG IFDStatusICC(PREADER_CONTEXT rContext, PDWORD pdwStatus,
 #ifndef PCSCLITE_STATIC_DRIVER
 	RESPONSECODE(*IFD_is_icc_present) (void) = NULL;
 	RESPONSECODE(*IFDH_icc_presence) (DWORD) = NULL;
-	RESPONSECODE(*IFD_get_capabilities) (DWORD, PUCHAR) = NULL;
+	RESPONSECODE(*IFD_get_capabilities) (DWORD, /*@out@*/ PUCHAR) = NULL;
 
 	if (rContext->dwVersion == IFD_HVERSION_1_0)
 	{
@@ -444,17 +433,14 @@ LONG IFDStatusICC(PREADER_CONTEXT rContext, PDWORD pdwStatus,
 		IFDH_icc_presence = rContext->psFunctions.psFunctions_v2.pvfICCPresence;
 #endif
 
-	/*
-	 * LOCK THIS CODE REGION
-	 */
-
-	SYS_MutexLock(rContext->mMutex);
+	/* LOCK THIS CODE REGION */ 
+	(void)SYS_MutexLock(rContext->mMutex);
 
 #ifndef PCSCLITE_STATIC_DRIVER
 	if (rContext->dwVersion == IFD_HVERSION_1_0)
 	{
 		ucValue[0] = rContext->dwSlot;
-		IFDSetCapabilities(rContext, TAG_IFD_SLOTNUM, 1, ucValue);
+		(void)IFDSetCapabilities(rContext, TAG_IFD_SLOTNUM, 1, ucValue);
 		rv = (*IFD_is_icc_present) ();
 	}
 	else
@@ -463,17 +449,15 @@ LONG IFDStatusICC(PREADER_CONTEXT rContext, PDWORD pdwStatus,
 	if (rContext->dwVersion == IFD_HVERSION_1_0)
 	{
 		ucValue[0] = rContext->dwSlot;
-		IFDSetCapabilities(rContext, TAG_IFD_SLOTNUM, 1, ucValue);
+		(void)IFDSetCapabilities(rContext, TAG_IFD_SLOTNUM, 1, ucValue);
 		rv = IFD_Is_ICC_Present();
 	}
 	else
 		rv = IFDHICCPresence(rContext->dwSlot);
 #endif
-	SYS_MutexUnLock(rContext->mMutex);
 
-	/*
-	 * END OF LOCKED REGION
-	 */
+	/* END OF LOCKED REGION */
+	(void)SYS_MutexUnLock(rContext->mMutex);
 
 	if (rv == IFD_SUCCESS || rv == IFD_ICC_PRESENT)
 		dwCardStatus |= SCARD_PRESENT;
@@ -487,7 +471,7 @@ LONG IFDStatusICC(PREADER_CONTEXT rContext, PDWORD pdwStatus,
 
 			if (rv == IFD_NO_SUCH_DEVICE)
 			{
-				SendHotplugSignal();
+				(void)SendHotplugSignal();
 				return SCARD_E_READER_UNAVAILABLE;
 			}
 
@@ -508,25 +492,20 @@ LONG IFDStatusICC(PREADER_CONTEXT rContext, PDWORD pdwStatus,
 
 			dwTag = TAG_IFD_ATR;
 
-			/*
-			 * LOCK THIS CODE REGION
-			 */
-
-			SYS_MutexLock(rContext->mMutex);
+			/* LOCK THIS CODE REGION */ 
+			(void)SYS_MutexLock(rContext->mMutex);
 
 			ucValue[0] = rContext->dwSlot;
-			IFDSetCapabilities(rContext, TAG_IFD_SLOTNUM, 1, ucValue);
+			(void)IFDSetCapabilities(rContext, TAG_IFD_SLOTNUM, 1, ucValue);
 
 #ifndef PCSCLITE_STATIC_DRIVER
 			rv = (*IFD_get_capabilities) (dwTag, pucAtr);
 #else
 			rv = IFD_Get_Capabilities(dwTag, pucAtr);
 #endif
-			SYS_MutexUnLock(rContext->mMutex);
 
-			/*
-			 * END OF LOCKED REGION
-			 */
+			/* END OF LOCKED REGION */
+			(void)SYS_MutexUnLock(rContext->mMutex);
 
 			/*
 			 * FIX :: This is a temporary way to return the correct size
@@ -576,7 +555,8 @@ LONG IFDControl_v2(PREADER_CONTEXT rContext, PUCHAR TxBuffer,
 	RESPONSECODE rv = IFD_SUCCESS;
 
 #ifndef PCSCLITE_STATIC_DRIVER
-	RESPONSECODE(*IFDH_control_v2) (DWORD, PUCHAR, DWORD, PUCHAR, PDWORD);
+	RESPONSECODE(*IFDH_control_v2) (DWORD, PUCHAR, DWORD, /*@out@*/ PUCHAR,
+		PDWORD);
 #endif
 
 	if (rContext->dwVersion != IFD_HVERSION_2_0)
@@ -586,10 +566,8 @@ LONG IFDControl_v2(PREADER_CONTEXT rContext, PUCHAR TxBuffer,
 	IFDH_control_v2 = rContext->psFunctions.psFunctions_v2.pvfControl;
 #endif
 
-	/*
-	 * LOCK THIS CODE REGION
-	 */
-	SYS_MutexLock(rContext->mMutex);
+	/* LOCK THIS CODE REGION */
+	(void)SYS_MutexLock(rContext->mMutex);
 
 #ifndef PCSCLITE_STATIC_DRIVER
 	rv = (*IFDH_control_v2) (rContext->dwSlot, TxBuffer, TxLength,
@@ -598,16 +576,17 @@ LONG IFDControl_v2(PREADER_CONTEXT rContext, PUCHAR TxBuffer,
 	rv = IFDHControl_v2(rContext->dwSlot, TxBuffer, TxLength,
 		RxBuffer, RxLength);
 #endif
-	SYS_MutexUnLock(rContext->mMutex);
-	/*
-	 * END OF LOCKED REGION
-	 */
+
+	/* END OF LOCKED REGION */
+	(void)SYS_MutexUnLock(rContext->mMutex);
 
 	if (rv == IFD_SUCCESS)
 		return SCARD_S_SUCCESS;
 	else
 	{
 		Log2(PCSC_LOG_ERROR, "Card not transacted: %ld", rv);
+		LogXxd(PCSC_LOG_DEBUG, "TxBuffer ", TxBuffer, TxLength);
+		LogXxd(PCSC_LOG_DEBUG, "RxBuffer ", RxBuffer, *RxLength);
 		return SCARD_E_NOT_TRANSACTED;
 	}
 }
@@ -638,11 +617,8 @@ LONG IFDControl(PREADER_CONTEXT rContext, DWORD ControlCode,
 	IFDH_control = rContext->psFunctions.psFunctions_v3.pvfControl;
 #endif
 
-	/*
-	 * LOCK THIS CODE REGION
-	 */
-
-	SYS_MutexLock(rContext->mMutex);
+	/* LOCK THIS CODE REGION */ 
+	(void)SYS_MutexLock(rContext->mMutex);
 
 #ifndef PCSCLITE_STATIC_DRIVER
 	rv = (*IFDH_control) (rContext->dwSlot, ControlCode, TxBuffer,
@@ -651,21 +627,23 @@ LONG IFDControl(PREADER_CONTEXT rContext, DWORD ControlCode,
 	rv = IFDHControl(rContext->dwSlot, ControlCode, TxBuffer,
 		TxLength, RxBuffer, RxLength, BytesReturned);
 #endif
-	SYS_MutexUnLock(rContext->mMutex);
 
-	/*
-	 * END OF LOCKED REGION
-	 */
+	/* END OF LOCKED REGION */
+	(void)SYS_MutexUnLock(rContext->mMutex);
 
 	if (rv == IFD_SUCCESS)
 		return SCARD_S_SUCCESS;
 	else
 	{
 		Log2(PCSC_LOG_ERROR, "Card not transacted: %ld", rv);
+		Log3(PCSC_LOG_DEBUG, "ControlCode: 0x%.8Lx BytesReturned: %ld",
+			ControlCode, *BytesReturned);
+		LogXxd(PCSC_LOG_DEBUG, "TxBuffer ", TxBuffer, TxLength);
+		LogXxd(PCSC_LOG_DEBUG, "RxBuffer ", RxBuffer, *BytesReturned);
 
 		if (rv == IFD_NO_SUCH_DEVICE)
 		{
-			SendHotplugSignal();
+			(void)SendHotplugSignal();
 			return SCARD_E_READER_UNAVAILABLE;
 		}
 
@@ -685,9 +663,9 @@ LONG IFDTransmit(PREADER_CONTEXT rContext, SCARD_IO_HEADER pioTxPci,
 
 #ifndef PCSCLITE_STATIC_DRIVER
 	RESPONSECODE(*IFD_transmit_to_icc) (SCARD_IO_HEADER, PUCHAR, DWORD,
-		PUCHAR, PDWORD, PSCARD_IO_HEADER) = NULL;
+		/*@out@*/ PUCHAR, PDWORD, PSCARD_IO_HEADER) = NULL;
 	RESPONSECODE(*IFDH_transmit_to_icc) (DWORD, SCARD_IO_HEADER, PUCHAR,
-		DWORD, PUCHAR, PDWORD, PSCARD_IO_HEADER) = NULL;
+		DWORD, /*@out@*/ PUCHAR, PDWORD, PSCARD_IO_HEADER) = NULL;
 #endif
 
 	/* log the APDU */
@@ -702,18 +680,14 @@ LONG IFDTransmit(PREADER_CONTEXT rContext, SCARD_IO_HEADER pioTxPci,
 			rContext->psFunctions.psFunctions_v2.pvfTransmitToICC;
 #endif
 
-	/*
-	 * LOCK THIS CODE REGION
-	 */
-
-	SYS_MutexLock(rContext->mMutex);
-
+	/* LOCK THIS CODE REGION */ 
+	(void)SYS_MutexLock(rContext->mMutex);
 
 #ifndef PCSCLITE_STATIC_DRIVER
 	if (rContext->dwVersion == IFD_HVERSION_1_0)
 	{
 		ucValue[0] = rContext->dwSlot;
-		IFDSetCapabilities(rContext, TAG_IFD_SLOTNUM, 1, ucValue);
+		(void)IFDSetCapabilities(rContext, TAG_IFD_SLOTNUM, 1, ucValue);
 		rv = (*IFD_transmit_to_icc) (pioTxPci, (LPBYTE) pucTxBuffer,
 			dwTxLength, pucRxBuffer, pdwRxLength, pioRxPci);
 	}
@@ -725,7 +699,7 @@ LONG IFDTransmit(PREADER_CONTEXT rContext, SCARD_IO_HEADER pioTxPci,
 	if (rContext->dwVersion == IFD_HVERSION_1_0)
 	{
 		ucValue[0] = rContext->dwSlot;
-		IFDSetCapabilities(rContext, TAG_IFD_SLOTNUM, 1, ucValue);
+		(void)IFDSetCapabilities(rContext, TAG_IFD_SLOTNUM, 1, ucValue);
 		rv = IFD_Transmit_to_ICC(pioTxPci, (LPBYTE) pucTxBuffer,
 			dwTxLength, pucRxBuffer, pdwRxLength, pioRxPci);
 	}
@@ -734,11 +708,9 @@ LONG IFDTransmit(PREADER_CONTEXT rContext, SCARD_IO_HEADER pioTxPci,
 			(LPBYTE) pucTxBuffer, dwTxLength,
 			pucRxBuffer, pdwRxLength, pioRxPci);
 #endif
-	SYS_MutexUnLock(rContext->mMutex);
 
-	/*
-	 * END OF LOCKED REGION
-	 */
+	/* END OF LOCKED REGION */
+	(void)SYS_MutexUnLock(rContext->mMutex);
 
 	/* log the returned status word */
 	DebugLogCategory(DEBUG_CATEGORY_SW, pucRxBuffer, *pdwRxLength);
@@ -751,7 +723,7 @@ LONG IFDTransmit(PREADER_CONTEXT rContext, SCARD_IO_HEADER pioTxPci,
 
 		if (rv == IFD_NO_SUCH_DEVICE)
 		{
-			SendHotplugSignal();
+			(void)SendHotplugSignal();
 			return SCARD_E_READER_UNAVAILABLE;
 		}
 
