@@ -5,7 +5,7 @@
  *  David Corcoran <corcoran@linuxnet.com>
  *  Ludovic Rousseau <ludovic.rouseau@free.fr>
  *
- * $Id: ifdwrapper.h 2151 2006-09-06 20:02:47Z rousseau $
+ * $Id: ifdwrapper.h 3260 2009-01-02 15:19:59Z rousseau $
  */
 
 /**
@@ -24,16 +24,18 @@ extern "C"
 
 	LONG IFDOpenIFD(PREADER_CONTEXT);
 	LONG IFDCloseIFD(PREADER_CONTEXT);
-	LONG IFDPowerICC(PREADER_CONTEXT, DWORD, PUCHAR, PDWORD);
-	LONG IFDStatusICC(PREADER_CONTEXT, PDWORD, PUCHAR, PDWORD);
-	LONG IFDControl_v2(PREADER_CONTEXT, PUCHAR, DWORD, PUCHAR, PDWORD);
+	LONG IFDPowerICC(PREADER_CONTEXT, DWORD, PUCHAR, /*@out@*/ PDWORD);
+	LONG IFDStatusICC(PREADER_CONTEXT, /*@out@*/ PDWORD, /*@out@*/ PUCHAR,
+		/*@out@*/ PDWORD);
+	LONG IFDControl_v2(PREADER_CONTEXT, PUCHAR, DWORD, /*@out@*/ PUCHAR,
+		PDWORD);
 	LONG IFDControl(PREADER_CONTEXT, DWORD, LPCVOID, DWORD, LPVOID,
 		DWORD, LPDWORD);
 	LONG IFDTransmit(PREADER_CONTEXT, SCARD_IO_HEADER,
-		PUCHAR, DWORD, PUCHAR, PDWORD, PSCARD_IO_HEADER);
+		PUCHAR, DWORD, /*@out@*/ PUCHAR, PDWORD, PSCARD_IO_HEADER);
 	LONG IFDSetPTS(PREADER_CONTEXT, DWORD, UCHAR, UCHAR, UCHAR, UCHAR);
 	LONG IFDSetCapabilities(PREADER_CONTEXT, DWORD, DWORD, PUCHAR);
-	LONG IFDGetCapabilities(PREADER_CONTEXT, DWORD, PDWORD, PUCHAR);
+	LONG IFDGetCapabilities(PREADER_CONTEXT, DWORD, PDWORD, /*@out@*/ PUCHAR);
 
 #ifdef __cplusplus
 }
