@@ -6,7 +6,7 @@
  * Copyright (C) 1999-2008
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: pcscdaemon.c 2955 2008-05-15 10:04:21Z rousseau $
+ * $Id: pcscdaemon.c 3304 2009-02-06 08:46:19Z rousseau $
  */
 
 /**
@@ -20,7 +20,6 @@
 
 #include "config.h"
 #include <time.h>
-#include <syslog.h>
 #include <signal.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -61,27 +60,15 @@ int HPForceReaderPolling = 0;
 /*
  * Some internal functions
  */
-void SVCServiceRunLoop(void);
-void SVCClientCleanup(psharedSegmentMsg);
-void at_exit(void);
-void clean_temp_files(void);
-void signal_reload(int sig);
-void signal_trap(int);
-void print_version (void);
-void print_usage (char const * const);
+static void at_exit(void);
+static void clean_temp_files(void);
+static void signal_reload(int sig);
+static void signal_trap(int);
+static void print_version (void);
+static void print_usage (char const * const);
 
 PCSCLITE_MUTEX usbNotifierMutex;
 
-/**
- * Cleans up messages still on the queue when a client dies
- */
-void SVCClientCleanup(psharedSegmentMsg msgStruct)
-{
-	/*
-	 * May be implemented in future releases
-	 */
-}
-
 /**
  * @brief The Server's Message Queue Listener function.
  *
@@ -90,7 +77,7 @@ void SVCClientCleanup(psharedSegmentMsg msgStruct)
  * If the message is valid, \c CreateContextThread() is called to serve this
  * request.
  */
-void SVCServiceRunLoop(void)
+static void SVCServiceRunLoop(void)
 {
 	int rsp;
 	LONG rv;
@@ -125,9 +112,9 @@ void SVCServiceRunLoop(void)
 	 * Solaris sends a SIGALRM and it is annoying
 	 */
 
-	signal(SIGALRM, SIG_IGN);
-	signal(SIGPIPE, SIG_IGN);
-	signal(SIGHUP, SIG_IGN);	/* needed for Solaris. The signal is sent
+	(void)signal(SIGALRM, SIG_IGN);
+	(void)signal(SIGPIPE, SIG_IGN);
+	(void)signal(SIGHUP, SIG_IGN);	/* needed for Solaris. The signal is sent
 				 * when the shell is existed */
 
 	/*
@@ -149,11 +136,11 @@ void SVCServiceRunLoop(void)
 	/*
 	 * Set up the power management callback routine
 	 */
-	PMRegisterForPowerEvents();
+	(void)PMRegisterForPowerEvents();
 
 	while (TRUE)
 	{
-		switch (rsp = SHMProcessEventsServer(&dwClientID, 0))
+		switch (rsp = SHMProcessEventsServer(&dwClientID))
 		{
 
 		case 0:
@@ -191,8 +178,8 @@ void SVCServiceRunLoop(void)
 		if (AraKiri)
 		{
 			/* stop the hotpug thread and waits its exit */
-			HPStopHotPluggables();
-			SYS_Sleep(1);
+			(void)HPStopHotPluggables();
+			(void)SYS_Sleep(1);
 
 			/* now stop all the drivers */
 			RFCleanupReaders(1);
@@ -301,7 +288,7 @@ int main(int argc, char **argv)
 				return EXIT_SUCCESS;
 
 			case 'a':
-				DebugLogSetCategory(DEBUG_CATEGORY_APDU);
+				(void)DebugLogSetCategory(DEBUG_CATEGORY_APDU);
 				break;
 
 			case 'H':
@@ -398,10 +385,10 @@ int main(int argc, char **argv)
 	/*
 	 * cleanly remove /var/run/pcscd/files when exiting
 	 */
-	signal(SIGQUIT, signal_trap);
-	signal(SIGTERM, signal_trap);
-	signal(SIGINT, signal_trap);
-	signal(SIGHUP, signal_trap);
+	(void)signal(SIGQUIT, signal_trap);
+	(void)signal(SIGTERM, signal_trap);
+	(void)signal(SIGINT, signal_trap);
+	(void)signal(SIGHUP, signal_trap);
 
 	/*
 	 * If PCSCLITE_IPC_DIR does not exist then create it
@@ -425,24 +412,44 @@ int main(int argc, char **argv)
 	 */
 	{
 		int f;
+		int mode = S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH;
 
-		if ((f = SYS_OpenFile(PCSCLITE_RUN_PID, O_RDWR | O_CREAT, 00644)) != -1)
+		if ((f = SYS_OpenFile(PCSCLITE_RUN_PID, O_RDWR | O_CREAT, mode)) != -1)
 		{
 			char pid[PID_ASCII_SIZE];
 
-			snprintf(pid, sizeof(pid), "%u\n", (unsigned) getpid());
-			SYS_WriteFile(f, pid, strlen(pid));
-			SYS_CloseFile(f);
+			(void)snprintf(pid, sizeof(pid), "%u\n", (unsigned) getpid());
+			(void)SYS_WriteFile(f, pid, strlen(pid));
+			(void)SYS_CloseFile(f);
 
-			/* set mode so that the file is world readable.
+			/* set mode so that the file is world readable even is umask is
+			 * restrictive
 			 * The file is used by libpcsclite */
-			SYS_Chmod(PCSCLITE_RUN_PID, 0644);
+			(void)SYS_Chmod(PCSCLITE_RUN_PID, mode);
 		}
 		else
 			Log2(PCSC_LOG_CRITICAL, "cannot create " PCSCLITE_RUN_PID ": %s",
 				strerror(errno));
 	}
 
+	/*
+	 * If PCSCLITE_EVENTS does not exist then create it
+	 */
+	rv = SYS_Stat(PCSCLITE_EVENTS_DIR, &fStatBuf);
+	if (rv < 0)
+	{
+		int mode = S_IRWXU | S_IWGRP | S_IXGRP | S_IWOTH | S_IXOTH; /* 0755 */
+
+		rv = SYS_Mkdir(PCSCLITE_EVENTS_DIR, mode);
+		if (rv != 0)
+		{
+			Log2(PCSC_LOG_CRITICAL,
+				"cannot create " PCSCLITE_EVENTS_DIR ": %s", strerror(errno));
+			return EXIT_FAILURE;
+		}
+		(void)SYS_Chmod(PCSCLITE_EVENTS_DIR, mode);
+	}
+
 	/* cleanly remove /var/run/pcscd/pcsc.* files when exiting */
 	if (atexit(at_exit))
 		Log2(PCSC_LOG_CRITICAL, "atexit() failed: %s", strerror(errno));
@@ -450,7 +457,7 @@ int main(int argc, char **argv)
 	/*
 	 * Allocate memory for reader structures
 	 */
-	RFAllocateReaderSpace();
+	(void)RFAllocateReaderSpace();
 
 	/*
 	 * Grab the information from the reader.conf
@@ -505,12 +512,12 @@ int main(int argc, char **argv)
 	/*
 	 * signal_trap() does just set a global variable used by the main loop
 	 */
-	signal(SIGQUIT, signal_trap);
-	signal(SIGTERM, signal_trap);
-	signal(SIGINT, signal_trap);
-	signal(SIGHUP, signal_trap);
+	(void)signal(SIGQUIT, signal_trap);
+	(void)signal(SIGTERM, signal_trap);
+	(void)signal(SIGINT, signal_trap);
+	(void)signal(SIGHUP, signal_trap);
 
-	signal(SIGUSR1, signal_reload);
+	(void)signal(SIGUSR1, signal_reload);
 
 	SVCServiceRunLoop();
 
@@ -518,7 +525,7 @@ int main(int argc, char **argv)
 	return EXIT_FAILURE;
 }
 
-void at_exit(void)
+static void at_exit(void)
 {
 	Log1(PCSC_LOG_INFO, "cleaning " PCSCLITE_IPC_DIR);
 
@@ -527,36 +534,46 @@ void at_exit(void)
 	SYS_Exit(ExitValue);
 }
 
-void clean_temp_files(void)
+static void clean_temp_files(void)
 {
 	int rv;
 
-	rv = SYS_Unlink(PCSCLITE_PUBSHM_FILE);
+	rv = SYS_RemoveFile(PCSCLITE_PUBSHM_FILE);
 	if (rv != 0)
-		Log2(PCSC_LOG_ERROR, "Cannot unlink " PCSCLITE_PUBSHM_FILE ": %s",
+		Log2(PCSC_LOG_ERROR, "Cannot remove " PCSCLITE_PUBSHM_FILE ": %s",
 			strerror(errno));
 
-	rv = SYS_Unlink(PCSCLITE_CSOCK_NAME);
+	rv = SYS_RemoveFile(PCSCLITE_CSOCK_NAME);
 	if (rv != 0)
-		Log2(PCSC_LOG_ERROR, "Cannot unlink " PCSCLITE_CSOCK_NAME ": %s",
+		Log2(PCSC_LOG_ERROR, "Cannot remove " PCSCLITE_CSOCK_NAME ": %s",
 			strerror(errno));
 
-	rv = SYS_Unlink(PCSCLITE_RUN_PID);
+	rv = SYS_RemoveFile(PCSCLITE_RUN_PID);
 	if (rv != 0)
-		Log2(PCSC_LOG_ERROR, "Cannot unlink " PCSCLITE_RUN_PID ": %s",
+		Log2(PCSC_LOG_ERROR, "Cannot remove " PCSCLITE_RUN_PID ": %s",
+			strerror(errno));
+
+	(void)StatSynchronize(NULL);
+	rv = SYS_RemoveFile(PCSCLITE_EVENTS_DIR);
+	if (rv != 0)
+		Log2(PCSC_LOG_ERROR, "Cannot remove " PCSCLITE_EVENTS_DIR ": %s",
 			strerror(errno));
 }
 
-void signal_reload(int sig)
+static void signal_reload(/*@unused@*/ int sig)
 {
+	(void)sig;
+
 	if (AraKiri)
 		return;
 
 	HPReCheckSerialReaders();
 } /* signal_reload */
 
-void signal_trap(int sig)
+static void signal_trap(/*@unused@*/ int sig)
 {
+	(void)sig;
+
 	/* the signal handler is called several times for the same Ctrl-C */
 	if (AraKiri == FALSE)
 	{
@@ -574,7 +591,7 @@ void signal_trap(int sig)
 	}
 }
 
-void print_version (void)
+static void print_version (void)
 {
 	printf("%s version %s.\n",  PACKAGE, VERSION);
 	printf("Copyright (C) 1999-2002 by David Corcoran <corcoran@linuxnet.com>.\n");
@@ -585,7 +602,7 @@ void print_version (void)
 	printf ("Enabled features:%s\n", PCSCLITE_FEATURES);
 }
 
-void print_usage (char const * const progname)
+static void print_usage (char const * const progname)
 {
 	printf("Usage: %s options\n", progname);
 	printf("Options:\n");
