@@ -4,7 +4,7 @@
  * Copyright (C) 2000-2003
  *  David Corcoran <corcoran@linuxnet.com>
  *
- * $Id: powermgt_generic.c 2261 2006-12-02 17:27:10Z rousseau $
+ * $Id: powermgt_generic.c 3412 2009-04-07 18:19:33Z rousseau $
  */
 
 /**
@@ -23,11 +23,7 @@
  * No, it's still not a perfect solution design wise.
  */
 
-#if !defined(__APPLE__)
-
 ULONG PMRegisterForPowerEvents(void)
 {
   return 0;
 }
-
-#endif
