@@ -6,7 +6,7 @@
  * Copyright (C) 2004
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: readerfactory.h 2543 2007-05-23 14:03:46Z rousseau $
+ * $Id: readerfactory.h 3466 2009-04-27 15:07:42Z rousseau $
  */
 
 /**
@@ -21,6 +21,7 @@
 
 #include "thread_generic.h"
 #include "ifdhandler.h"
+#include "pcscd.h"
 
 #ifdef __cplusplus
 extern "C"
@@ -29,10 +30,10 @@ extern "C"
 
 	typedef struct
 	{
-		char *pcFriendlyname;
-		char *pcDevicename;
-		char *pcLibpath;
-		int dwChannelId;
+		char *pcFriendlyname;	/**< FRIENDLYNAME */
+		char *pcDevicename;		/**< DEVICENAME */
+		char *pcLibpath;		/**< LIBPATH */
+		int dwChannelId;		/**< CHANNELID */
 	} SerialReader;
 
 	struct FctMap_V1
@@ -93,91 +94,46 @@ extern "C"
 
 	typedef struct FctMap_V3 FCT_MAP_V3, *PFCT_MAP_V3;
 
-	/*
-	 * The following is not currently used but in place if needed
-	 */
-
-	struct RdrCapabilities
-	{
-		DWORD dwAsynch_Supported;	/* Asynchronous Support */
-		DWORD dwDefault_Clock;	/* Default Clock Rate */
-		DWORD dwMax_Clock;		/* Max Clock Rate */
-		DWORD dwDefault_Data_Rate;	/* Default Data Rate */
-		DWORD dwMax_Data_Rate;	/* Max Data Rate */
-		DWORD dwMax_IFSD;		/* Maximum IFSD Size */
-		DWORD dwSynch_Supported;	/* Synchronous Support */
-		DWORD dwPower_Mgmt;		/* Power Mgmt Features */
-		DWORD dwCard_Auth_Devices;	/* Card Auth Devices */
-		DWORD dwUser_Auth_Device;	/* User Auth Devices */
-		DWORD dwMechanics_Supported;	/* Machanics Supported */
-		DWORD dwVendor_Features;	/* User Defined.  */
-	};
-
-	typedef struct RdrCapabilities RDR_CAPABILITIES, *PRDR_CAPABILITIES;
-
-	struct ProtOptions
-	{
-		DWORD dwProtocol_Type;	/* Protocol Type */
-		DWORD dwCurrent_Clock;	/* Current Clock */
-		DWORD dwCurrent_F;		/* Current F */
-		DWORD dwCurrent_D;		/* Current D */
-		DWORD dwCurrent_N;		/* Current N */
-		DWORD dwCurrent_W;		/* Current W */
-		DWORD dwCurrent_IFSC;	/* Current IFSC */
-		DWORD dwCurrent_IFSD;	/* Current IFSD */
-		DWORD dwCurrent_BWT;	/* Current BWT */
-		DWORD dwCurrent_CWT;	/* Current CWT */
-		DWORD dwCurrent_EBC;	/* Current EBC */
-	};
-
-	typedef struct ProtOptions PROT_OPTIONS, *PPROT_OPTIONS;
-
 	struct RdrCliHandles
 	{
-		SCARDHANDLE hCard;		/* hCard for this connection */
-		DWORD dwEventStatus;	/* Recent event that must be sent */
+		SCARDHANDLE hCard;		/**< hCard for this connection */
+		DWORD dwEventStatus;	/**< Recent event that must be sent */
 	};
 
 	typedef struct RdrCliHandles RDR_CLIHANDLES, *PRDR_CLIHANDLES;
 
 	struct ReaderContext
 	{
-		char lpcReader[MAX_READERNAME];	/* Reader Name */
-		char lpcLibrary[MAX_LIBNAME];	/* Library Path */
-		char lpcDevice[MAX_DEVICENAME];	/* Device Name */
-		PCSCLITE_THREAD_T pthThread;	/* Event polling thread */
-		PCSCLITE_MUTEX_T mMutex;	/* Mutex for this connection */
+		char lpcReader[MAX_READERNAME];	/**< Reader Name */
+		char lpcLibrary[MAX_LIBNAME];	/**< Library Path */
+		char lpcDevice[MAX_DEVICENAME];	/**< Device Name */
+		PCSCLITE_THREAD_T pthThread;	/**< Event polling thread */
+		RESPONSECODE (*pthCardEvent)(DWORD);	/**< Card Event sync */
+		PCSCLITE_MUTEX_T mMutex;	/**< Mutex for this connection */
 		RDR_CLIHANDLES psHandles[PCSCLITE_MAX_READER_CONTEXT_CHANNELS];
-                                         /* Structure of connected handles */
+                                         /**< Structure of connected handles */
 		union
 		{
-			FCT_MAP_V1 psFunctions_v1;	/* API V1.0 */
-			FCT_MAP_V2 psFunctions_v2;	/* API V2.0 */
-			FCT_MAP_V3 psFunctions_v3;	/* API V3.0 */
-		} psFunctions;
+			FCT_MAP_V1 psFunctions_v1;	/**< API V1.0 */
+			FCT_MAP_V2 psFunctions_v2;	/**< API V2.0 */
+			FCT_MAP_V3 psFunctions_v3;	/**< API V3.0 */
+		} psFunctions;	/**< driver functions */
 
-		LPVOID vHandle;			/* Dlopen handle */
-		DWORD dwVersion;		/* IFD Handler version number */
-		DWORD dwPort;			/* Port ID */
-		DWORD dwSlot;			/* Current Reader Slot */
-		DWORD dwBlockStatus;	/* Current blocking status */
-		DWORD dwLockId;			/* Lock Id */
-		int LockCount;			/* number of recursive locks */
-		DWORD dwIdentity;		/* Shared ID High Nibble */
-		int32_t dwContexts;		/* Number of open contexts */
-		PDWORD pdwFeeds;		/* Number of shared client to lib */
-		PDWORD pdwMutex;		/* Number of client to mutex */
+		LPVOID vHandle;			/**< Dlopen handle */
+		DWORD dwVersion;		/**< IFD Handler version number */
+		DWORD dwPort;			/**< Port ID */
+		DWORD dwSlot;			/**< Current Reader Slot */
+		DWORD dwBlockStatus;	/**< Current blocking status */
+		DWORD dwLockId;			/**< Lock Id */
+		int LockCount;			/**< number of recursive locks */
+		DWORD dwIdentity;		/**< Shared ID High Nibble */
+		int32_t dwContexts;		/**< Number of open contexts */
+		PDWORD pdwFeeds;		/**< Number of shared client to lib */
+		PDWORD pdwMutex;		/**< Number of client to mutex */
 
-		struct pubReaderStatesList *readerState; /* link to the reader state */
+		struct pubReaderStatesList *readerState; /**< link to the reader state */
 		/* we can't use PREADER_STATE here since eventhandler.h can't be
 		 * included because of circular dependencies */
-
-		/* these structures are unused */
-#if 0
-		RDR_CAPABILITIES psCapabilites;	/* Structure of reader
-						   capabilities */
-		PROT_OPTIONS psProtOptions;	/* Structure of protocol options */
-#endif
 	};
 
 	typedef struct ReaderContext READER_CONTEXT, *PREADER_CONTEXT;
@@ -187,12 +143,13 @@ extern "C"
 	LONG RFRemoveReader(LPSTR, DWORD);
 	LONG RFSetReaderName(PREADER_CONTEXT, LPSTR, LPSTR, DWORD, DWORD);
 	LONG RFListReaders(LPSTR, LPDWORD);
-	LONG RFReaderInfo(LPSTR, struct ReaderContext **);
-	LONG RFReaderInfoNamePort(DWORD, LPSTR, struct ReaderContext **);
-	LONG RFReaderInfoById(DWORD, struct ReaderContext **);
+	LONG RFReaderInfo(LPSTR, /*@out@*/ struct ReaderContext **);
+	LONG RFReaderInfoNamePort(DWORD, LPSTR, /*@out@*/ struct ReaderContext **);
+	LONG RFReaderInfoById(DWORD, /*@out@*/ struct ReaderContext **);
 	LONG RFCheckSharing(DWORD);
 	LONG RFLockSharing(DWORD);
 	LONG RFUnlockSharing(DWORD);
+	LONG RFUnlockAllSharing(DWORD);
 	LONG RFUnblockReader(PREADER_CONTEXT);
 	LONG RFUnblockContext(SCARDCONTEXT);
 	LONG RFLoadReader(PREADER_CONTEXT);
