@@ -6,7 +6,7 @@
  * Copyright (C) 2004-2008
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: testpcsc.c 3021 2008-06-24 09:59:35Z rousseau $
+ * $Id: testpcsc.c 3304 2009-02-06 08:46:19Z rousseau $
  */
 
 /**
@@ -35,8 +35,7 @@
 #define NORMAL "\33[0m"
 #define MAGENTA "\33[35m"
 
-void test_rv(LONG rv, SCARDCONTEXT hContext, int dont_panic);
-void test_rv(LONG rv, SCARDCONTEXT hContext, int dont_panic)
+static void test_rv(LONG rv, SCARDCONTEXT hContext, int dont_panic)
 {
 	if (rv != SCARD_S_SUCCESS)
 	{
@@ -45,23 +44,29 @@ void test_rv(LONG rv, SCARDCONTEXT hContext, int dont_panic)
 		else
 		{
 			printf(RED "%s\n" NORMAL, pcsc_stringify_error(rv));
-			SCardReleaseContext(hContext);
+			(void)SCardReleaseContext(hContext);
 			exit(-1);
 		}
 	}
 	else
-		puts(pcsc_stringify_error(rv));
+		(void)puts(pcsc_stringify_error(rv));
 }
 
-int main(int argc, char **argv)
+int main(/*@unused@*/ int argc, /*@unused@*/ char **argv)
 {
 	SCARDHANDLE hCard;
 	SCARDCONTEXT hContext;
 	SCARD_READERSTATE_A rgReaderStates[1];
 	DWORD dwReaderLen, dwState, dwProt, dwAtrLen;
 	DWORD dwPref, dwReaders = 0;
-	char *pcReaders, *mszReaders;
+	char *pcReaders = NULL, *mszReaders;
+#ifdef USE_AUTOALLOCATE
+	unsigned char *pbAtr = NULL;
+#else
 	unsigned char pbAtr[MAX_ATR_SIZE];
+#endif
+	unsigned char buf[100];
+	DWORD dwBufLen;
 	unsigned char *pbAttr = NULL;
 	DWORD pcbAttrLen;
 	char *mszGroups;
@@ -76,6 +81,9 @@ int main(int argc, char **argv)
 	unsigned char bRecvBuffer[MAX_BUFFER_SIZE];
 	DWORD send_length, length;
 
+	(void)argc;
+	(void)argv;
+
 	printf("\nMUSCLE PC/SC Lite unitary test Program\n\n");
 
 	printf(MAGENTA "THIS PROGRAM IS NOT DESIGNED AS A TESTING TOOL FOR END USERS!\n");
@@ -93,12 +101,6 @@ int main(int argc, char **argv)
 	rv = SCardIsValidContext(hContext+1);
 	test_rv(rv, hContext, DONT_PANIC);
 
-	printf("Testing SCardGetStatusChange \n");
-	printf("Please insert a working reader\t: ");
-	fflush(stdout);
-	rv = SCardGetStatusChange(hContext, INFINITE, 0, 0);
-	test_rv(rv, hContext, PANIC);
-
 	printf("Testing SCardListReaderGroups\t: ");
 #ifdef USE_AUTOALLOCATE
 	dwGroups = SCARD_AUTOALLOCATE;
@@ -133,9 +135,23 @@ int main(int argc, char **argv)
 #endif
 
 wait_for_card_again:
-	printf("Testing SCardListReaders\t: ");
-
 	mszGroups = NULL;
+	printf("Testing SCardListReaders\t: ");
+	rv = SCardListReaders(hContext, mszGroups, NULL, &dwReaders);
+	test_rv(rv, hContext, DONT_PANIC);
+	if (SCARD_E_NO_READERS_AVAILABLE == rv)
+	{
+		printf("Testing SCardGetStatusChange \n");
+		printf("Please insert a working reader\t: ");
+		(void)fflush(stdout);
+		rgReaderStates[0].szReader = "\\\\?PnP?\\Notification";
+		rgReaderStates[0].dwCurrentState = SCARD_STATE_EMPTY;
+
+		rv = SCardGetStatusChange(hContext, INFINITE, rgReaderStates, 1);
+		test_rv(rv, hContext, PANIC);
+	}
+
+	printf("Testing SCardListReaders\t: ");
 #ifdef USE_AUTOALLOCATE
 	dwReaders = SCARD_AUTOALLOCATE;
 	rv = SCardListReaders(hContext, mszGroups, (LPSTR)&mszReaders, &dwReaders);
@@ -147,7 +163,7 @@ wait_for_card_again:
 	mszReaders = calloc(dwReaders, sizeof(char));
 	rv = SCardListReaders(hContext, mszGroups, mszReaders, &dwReaders);
 #endif
-	test_rv(rv, hContext, PANIC);
+	test_rv(rv, hContext, DONT_PANIC);
 
 	/*
 	 * Have to understand the multi-string here
@@ -167,8 +183,8 @@ wait_for_card_again:
 			char input[80];
 
 			printf("Enter the reader number\t\t: ");
-			fgets(input, sizeof(input), stdin);
-			sscanf(input, "%d", &iReader);
+			(void)fgets(input, sizeof(input), stdin);
+			(void)sscanf(input, "%d", &iReader);
 
 			if (iReader > p || iReader <= 0)
 				printf("Invalid Value - try again\n");
@@ -181,10 +197,10 @@ wait_for_card_again:
 	rgReaderStates[0].dwCurrentState = SCARD_STATE_EMPTY;
 
 	printf("Waiting for card insertion\t: ");
-	fflush(stdout);
+	(void)fflush(stdout);
 	rv = SCardGetStatusChange(hContext, INFINITE, rgReaderStates, 1);
 	test_rv(rv, hContext, PANIC);
-	if (SCARD_STATE_EMPTY == rgReaderStates[0].dwEventState)
+	if (rgReaderStates[0].dwEventState & SCARD_STATE_UNKNOWN)
 	{
 		printf("\nA reader has been connected/disconnected\n");
 		goto wait_for_card_again;
@@ -289,32 +305,32 @@ wait_for_card_again:
 #endif
 
 	printf("Testing SCardGetAttrib\t\t: ");
-	dwAtrLen = sizeof(pbAtr);
-	rv = SCardGetAttrib(hCard, SCARD_ATTR_VENDOR_IFD_VERSION, pbAtr, &dwAtrLen);
+	dwBufLen = sizeof(buf);
+	rv = SCardGetAttrib(hCard, SCARD_ATTR_VENDOR_IFD_VERSION, buf, &dwBufLen);
 	test_rv(rv, hContext, DONT_PANIC);
 	if (rv == SCARD_S_SUCCESS)
 		printf("Vendor IFD version\t\t: " GREEN "0x%08lX\n" NORMAL,
-			((DWORD *)pbAtr)[0]);
+			((DWORD *)buf)[0]);
 
 	printf("Testing SCardGetAttrib\t\t: ");
-	dwAtrLen = sizeof(pbAtr);
-	rv = SCardGetAttrib(hCard, SCARD_ATTR_MAXINPUT, pbAtr, &dwAtrLen);
+	dwBufLen = sizeof(buf);
+	rv = SCardGetAttrib(hCard, SCARD_ATTR_MAXINPUT, buf, &dwBufLen);
 	test_rv(rv, hContext, DONT_PANIC);
 	if (rv == SCARD_S_SUCCESS)
 	{
-		if (dwAtrLen == sizeof(uint32_t))
+		if (dwBufLen == sizeof(uint32_t))
 			printf("Max message length\t\t: " GREEN "%d\n" NORMAL,
-				*(uint32_t *)pbAtr);
+				*(uint32_t *)buf);
 		else
 			printf(RED "Wrong size" NORMAL);
 	}
 
 	printf("Testing SCardGetAttrib\t\t: ");
-	dwAtrLen = sizeof(pbAtr);
-	rv = SCardGetAttrib(hCard, SCARD_ATTR_VENDOR_NAME, pbAtr, &dwAtrLen);
+	dwBufLen = sizeof(buf);
+	rv = SCardGetAttrib(hCard, SCARD_ATTR_VENDOR_NAME, buf, &dwBufLen);
 	test_rv(rv, hContext, DONT_PANIC);
 	if (rv == SCARD_S_SUCCESS)
-		printf("Vendor name\t\t\t: " GREEN "%s\n" NORMAL, pbAtr);
+		printf("Vendor name\t\t\t: " GREEN "%s\n" NORMAL, buf);
 
 	printf("Testing SCardSetAttrib\t\t: ");
 	rv = SCardSetAttrib(hCard, SCARD_ATTR_ATR_STRING, (LPCBYTE)"", 1);
@@ -322,12 +338,19 @@ wait_for_card_again:
 
 	printf("Testing SCardStatus\t\t: ");
 
+#ifdef USE_AUTOALLOCATE
+	dwReaderLen = SCARD_AUTOALLOCATE;
+	dwAtrLen = SCARD_AUTOALLOCATE;
+	rv = SCardStatus(hCard, (LPSTR)&pcReaders, &dwReaderLen, &dwState, &dwProt,
+		(LPBYTE)&pbAtr, &dwAtrLen);
+#else
 	dwReaderLen = 100;
 	pcReaders   = malloc(sizeof(char) * 100);
 	dwAtrLen    = MAX_ATR_SIZE;
 
 	rv = SCardStatus(hCard, pcReaders, &dwReaderLen, &dwState, &dwProt,
 		pbAtr, &dwAtrLen);
+#endif
 	test_rv(rv, hContext, PANIC);
 
 	printf("Current Reader Name\t\t: " GREEN "%s\n" NORMAL, pcReaders);
@@ -343,14 +366,26 @@ wait_for_card_again:
 	}
 	printf(NORMAL "\n");
 
+#ifdef USE_AUTOALLOCATE
+	printf("Testing SCardFreeMemory\t\t: ");
+	rv = SCardFreeMemory(hContext, pcReaders);
+	test_rv(rv, hContext, PANIC);
+	printf("Testing SCardFreeMemory\t\t: ");
+	rv = SCardFreeMemory(hContext, pbAtr);
+	test_rv(rv, hContext, PANIC);
+#else
+	if (pcReaders)
+		free(pcReaders);
+#endif
+
 	if (rv != SCARD_S_SUCCESS)
 	{
-		SCardDisconnect(hCard, SCARD_RESET_CARD);
-		SCardReleaseContext(hContext);
+		(void)SCardDisconnect(hCard, SCARD_RESET_CARD);
+		(void)SCardReleaseContext(hContext);
 	}
 
 	printf("Press enter: ");
-	getchar();
+	(void)getchar();
 	printf("Testing SCardReconnect\t\t: ");
 	rv = SCardReconnect(hCard, SCARD_SHARE_SHARED,
 		SCARD_PROTOCOL_T0 | SCARD_PROTOCOL_T1, SCARD_UNPOWER_CARD, &dwPref);
