@@ -5,7 +5,7 @@
  *  David Corcoran <corcoran@linuxnet.com>
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: winscard.c 2661 2007-10-18 09:13:16Z rousseau $
+ * $Id: winscard.c 3467 2009-04-27 15:10:48Z rousseau $
  */
 
 /**
@@ -93,6 +93,7 @@
 #include "atrhandler.h"
 #include "sys_generic.h"
 #include "eventhandler.h"
+#include "utils.h"
 
 #undef DO_PROFILE
 #ifdef DO_PROFILE
@@ -200,12 +201,10 @@ SCARD_IO_REQUEST g_rgSCardRawPci = { SCARD_PROTOCOL_RAW, sizeof(SCARD_IO_REQUEST
  *
  * @param[in] dwScope Scope of the establishment.
  * This can either be a local or remote connection.
- * <ul>
- *   <li>SCARD_SCOPE_USER - Not used.
- *   <li>SCARD_SCOPE_TERMINAL - Not used.
- *   <li>SCARD_SCOPE_GLOBAL - Not used.
- *   <li>SCARD_SCOPE_SYSTEM - Services on the local machine.
- * </ul>
+ * - SCARD_SCOPE_USER - Not used.
+ * - SCARD_SCOPE_TERMINAL - Not used.
+ * - SCARD_SCOPE_GLOBAL - Not used.
+ * - SCARD_SCOPE_SYSTEM - Services on the local machine.
  * @param[in] pvReserved1 Reserved for future use. Can be used for remote connection.
  * @param[in] pvReserved2 Reserved for future use.
  * @param[out] phContext Returned Application Context.
@@ -215,9 +214,11 @@ SCARD_IO_REQUEST g_rgSCardRawPci = { SCARD_PROTOCOL_RAW, sizeof(SCARD_IO_REQUEST
  * @retval SCARD_E_INVALID_VALUE Invalid scope type passed (\ref SCARD_E_INVALID_VALUE)
  * @retval SCARD_E_INVALID_PARAMETER phContext is null (\ref SCARD_E_INVALID_PARAMETER)
  */
-LONG SCardEstablishContext(DWORD dwScope, LPCVOID pvReserved1,
-	LPCVOID pvReserved2, LPSCARDCONTEXT phContext)
+LONG SCardEstablishContext(DWORD dwScope, /*@unused@*/ LPCVOID pvReserved1,
+	/*@unused@*/ LPCVOID pvReserved2, LPSCARDCONTEXT phContext)
 {
+	(void)pvReserved1;
+	(void)pvReserved2;
 	/*
 	 * Check for NULL pointer
 	 */
@@ -227,7 +228,6 @@ LONG SCardEstablishContext(DWORD dwScope, LPCVOID pvReserved1,
 	if (dwScope != SCARD_SCOPE_USER && dwScope != SCARD_SCOPE_TERMINAL &&
 		dwScope != SCARD_SCOPE_SYSTEM && dwScope != SCARD_SCOPE_GLOBAL)
 	{
-
 		*phContext = 0;
 		return SCARD_E_INVALID_VALUE;
 	}
@@ -255,17 +255,20 @@ LONG SCardReleaseContext(SCARDCONTEXT hContext)
 	return SCARD_S_SUCCESS;
 }
 
-LONG SCardSetTimeout(SCARDCONTEXT hContext, DWORD dwTimeout)
+LONG SCardSetTimeout(/*@unused@*/ SCARDCONTEXT hContext,
+	/*@unused@*/ DWORD dwTimeout)
 {
 	/*
 	 * This is only used at the client side of an RPC call but just in
 	 * case someone calls it here
 	 */
 
+	(void)hContext;
+	(void)dwTimeout;
 	return SCARD_E_UNSUPPORTED_FEATURE;
 }
 
-LONG SCardConnect(SCARDCONTEXT hContext, LPCSTR szReader,
+LONG SCardConnect(/*@unused@*/ SCARDCONTEXT hContext, LPCSTR szReader,
 	DWORD dwShareMode, DWORD dwPreferredProtocols, LPSCARDHANDLE phCard,
 	LPDWORD pdwActiveProtocol)
 {
@@ -273,6 +276,7 @@ LONG SCardConnect(SCARDCONTEXT hContext, LPCSTR szReader,
 	PREADER_CONTEXT rContext = NULL;
 	DWORD dwStatus;
 
+	(void)hContext;
 	PROFILE_START
 
 	/*
@@ -283,7 +287,8 @@ LONG SCardConnect(SCARDCONTEXT hContext, LPCSTR szReader,
 	else
 		*phCard = 0;
 
-	if (!(dwPreferredProtocols & SCARD_PROTOCOL_T0) &&
+	if ((dwShareMode != SCARD_SHARE_DIRECT) &&
+			!(dwPreferredProtocols & SCARD_PROTOCOL_T0) &&
 			!(dwPreferredProtocols & SCARD_PROTOCOL_T1) &&
 			!(dwPreferredProtocols & SCARD_PROTOCOL_RAW) &&
 			!(dwPreferredProtocols & SCARD_PROTOCOL_ANY_OLD))
@@ -334,11 +339,8 @@ LONG SCardConnect(SCARDCONTEXT hContext, LPCSTR szReader,
 	{
 		Log1(PCSC_LOG_INFO, "Waiting for release of lock");
 		while (rContext->dwLockId != 0)
-			SYS_USleep(PCSCLITE_LOCK_POLL_RATE);
+			(void)SYS_USleep(PCSCLITE_LOCK_POLL_RATE);
 		Log1(PCSC_LOG_INFO, "Lock released");
-
-		/* Allow the status thread to convey information */
-		SYS_USleep(PCSCLITE_STATUS_POLL_RATE + 10);
 	}
 
 	/* the reader has been removed while we were waiting */
@@ -381,6 +383,10 @@ LONG SCardConnect(SCARDCONTEXT hContext, LPCSTR szReader,
 	{
 		if (dwShareMode != SCARD_SHARE_DIRECT)
 		{
+			/* lock here instead in IFDSetPTS() to lock up to
+			 * setting rContext->readerState->cardProtocol */
+			(void)SYS_MutexLock(rContext->mMutex);
+
 			/* the protocol is not yet set (no PPS yet) */
 			if (SCARD_PROTOCOL_UNDEFINED == rContext->readerState->cardProtocol)
 			{
@@ -404,16 +410,26 @@ LONG SCardConnect(SCARDCONTEXT hContext, LPCSTR szReader,
 
 				/* keep cardProtocol = SCARD_PROTOCOL_UNDEFINED in case of error  */
 				if (SET_PROTOCOL_PPS_FAILED == ret)
+				{
+					(void)SYS_MutexUnLock(rContext->mMutex);
 					return SCARD_W_UNRESPONSIVE_CARD;
+				}
 
 				if (SET_PROTOCOL_WRONG_ARGUMENT == ret)
+				{
+					(void)SYS_MutexUnLock(rContext->mMutex);
 					return SCARD_E_PROTO_MISMATCH;
+				}
 
 				/* use negociated protocol */
 				rContext->readerState->cardProtocol = ret;
+
+				(void)SYS_MutexUnLock(rContext->mMutex);
 			}
 			else
 			{
+				(void)SYS_MutexUnLock(rContext->mMutex);
+
 				if (! (dwPreferredProtocols & rContext->readerState->cardProtocol))
 					return SCARD_E_PROTO_MISMATCH;
 			}
@@ -454,11 +470,11 @@ LONG SCardConnect(SCARDCONTEXT hContext, LPCSTR szReader,
 		if (rContext->dwContexts == SCARD_NO_CONTEXT)
 		{
 			rContext->dwContexts = SCARD_EXCLUSIVE_CONTEXT;
-			RFLockSharing(*phCard);
+			(void)RFLockSharing(*phCard);
 		}
 		else
 		{
-			RFDestroyReaderHandle(*phCard);
+			(void)RFDestroyReaderHandle(*phCard);
 			*phCard = 0;
 			return SCARD_E_SHARING_VIOLATION;
 		}
@@ -481,7 +497,7 @@ LONG SCardConnect(SCARDCONTEXT hContext, LPCSTR szReader,
 		/*
 		 * Clean up - there is no more room
 		 */
-		RFDestroyReaderHandle(*phCard);
+		(void)RFDestroyReaderHandle(*phCard);
 		if (rContext->dwContexts == SCARD_EXCLUSIVE_CONTEXT)
 			rContext->dwContexts = SCARD_NO_CONTEXT;
 		else
@@ -496,9 +512,10 @@ LONG SCardConnect(SCARDCONTEXT hContext, LPCSTR szReader,
 	}
 
 	/*
-	 * Allow the status thread to convey information
+	 * Propagate new state to Shared Memory
 	 */
-	SYS_USleep(PCSCLITE_STATUS_POLL_RATE + 10);
+	rContext->readerState->readerSharing = rContext->dwContexts;
+	(void)StatSynchronize(rContext->readerState);
 
 	PROFILE_END
 
@@ -511,7 +528,6 @@ LONG SCardReconnect(SCARDHANDLE hCard, DWORD dwShareMode,
 {
 	LONG rv;
 	PREADER_CONTEXT rContext = NULL;
-	int do_sleep = 1;
 
 	Log1(PCSC_LOG_DEBUG, "Attempting reconnect to token.");
 
@@ -531,7 +547,8 @@ LONG SCardReconnect(SCARDHANDLE hCard, DWORD dwShareMode,
 			dwShareMode != SCARD_SHARE_DIRECT)
 		return SCARD_E_INVALID_VALUE;
 
-	if (!(dwPreferredProtocols & SCARD_PROTOCOL_T0) &&
+	if ((dwShareMode != SCARD_SHARE_DIRECT) &&
+			!(dwPreferredProtocols & SCARD_PROTOCOL_T0) &&
 			!(dwPreferredProtocols & SCARD_PROTOCOL_T1) &&
 			!(dwPreferredProtocols & SCARD_PROTOCOL_RAW) &&
 			!(dwPreferredProtocols & SCARD_PROTOCOL_ANY_OLD))
@@ -621,7 +638,7 @@ LONG SCardReconnect(SCARDHANDLE hCard, DWORD dwShareMode,
 				/*
 				 * Notify the card has been reset
 				 */
-				RFSetReaderEventState(rContext, SCARD_RESET);
+				(void)RFSetReaderEventState(rContext, SCARD_RESET);
 
 				/*
 				 * Set up the status bit masks on dwStatus
@@ -661,7 +678,7 @@ LONG SCardReconnect(SCARDHANDLE hCard, DWORD dwShareMode,
 					UCHAR ucAtr[MAX_ATR_SIZE];
 
 					Log1(PCSC_LOG_ERROR, "Error resetting card.");
-					IFDStatusICC(rContext, &dwStatus, ucAtr, &dwAtrLen2);
+					(void)IFDStatusICC(rContext, &dwStatus, ucAtr, &dwAtrLen2);
 					if (dwStatus & SCARD_PRESENT)
 						return SCARD_W_UNRESPONSIVE_CARD;
 					else
@@ -675,8 +692,6 @@ LONG SCardReconnect(SCARDHANDLE hCard, DWORD dwShareMode,
 				return SCARD_F_INTERNAL_ERROR;
 				break;
 		}
-
-		do_sleep = 1;
 	}
 	else
 		if (dwInitialization == SCARD_LEAVE_CARD)
@@ -684,7 +699,6 @@ LONG SCardReconnect(SCARDHANDLE hCard, DWORD dwShareMode,
 			/*
 			 * Do nothing
 			 */
-			do_sleep = 0;
 		}
 
 	/*******************************************
@@ -699,6 +713,10 @@ LONG SCardReconnect(SCARDHANDLE hCard, DWORD dwShareMode,
 	{
 		if (dwShareMode != SCARD_SHARE_DIRECT)
 		{
+			/* lock here instead in IFDSetPTS() to lock up to
+			 * setting rContext->readerState->cardProtocol */
+			(void)SYS_MutexLock(rContext->mMutex);
+
 			/* the protocol is not yet set (no PPS yet) */
 			if (SCARD_PROTOCOL_UNDEFINED == rContext->readerState->cardProtocol)
 			{
@@ -720,16 +738,26 @@ LONG SCardReconnect(SCARDHANDLE hCard, DWORD dwShareMode,
 
 				/* keep cardProtocol = SCARD_PROTOCOL_UNDEFINED in case of error  */
 				if (SET_PROTOCOL_PPS_FAILED == ret)
+				{
+					(void)SYS_MutexUnLock(rContext->mMutex);
 					return SCARD_W_UNRESPONSIVE_CARD;
+				}
 
 				if (SET_PROTOCOL_WRONG_ARGUMENT == ret)
+				{
+					(void)SYS_MutexUnLock(rContext->mMutex);
 					return SCARD_E_PROTO_MISMATCH;
+				}
 
 				/* use negociated protocol */
 				rContext->readerState->cardProtocol = ret;
+
+				(void)SYS_MutexUnLock(rContext->mMutex);
 			}
 			else
 			{
+				(void)SYS_MutexUnLock(rContext->mMutex);
+
 				if (! (dwPreferredProtocols & rContext->readerState->cardProtocol))
 					return SCARD_E_PROTO_MISMATCH;
 			}
@@ -750,7 +778,7 @@ LONG SCardReconnect(SCARDHANDLE hCard, DWORD dwShareMode,
 			if (rContext->dwContexts == SCARD_LAST_CONTEXT)
 			{
 				rContext->dwContexts = SCARD_EXCLUSIVE_CONTEXT;
-				RFLockSharing(hCard);
+				(void)RFLockSharing(hCard);
 			} else
 			{
 				return SCARD_E_SHARING_VIOLATION;
@@ -768,7 +796,7 @@ LONG SCardReconnect(SCARDHANDLE hCard, DWORD dwShareMode,
 			/*
 			 * We are in exclusive mode but want to share now
 			 */
-			RFUnlockSharing(hCard);
+			(void)RFUnlockSharing(hCard);
 			rContext->dwContexts = SCARD_LAST_CONTEXT;
 		}
 	} else if (dwShareMode == SCARD_SHARE_DIRECT)
@@ -783,7 +811,7 @@ LONG SCardReconnect(SCARDHANDLE hCard, DWORD dwShareMode,
 			/*
 			 * We are in exclusive mode but want to share now
 			 */
-			RFUnlockSharing(hCard);
+			(void)RFUnlockSharing(hCard);
 			rContext->dwContexts = SCARD_LAST_CONTEXT;
 		}
 	} else
@@ -792,13 +820,13 @@ LONG SCardReconnect(SCARDHANDLE hCard, DWORD dwShareMode,
 	/*
 	 * Clear a previous event to the application
 	 */
-	RFClearReaderEventState(rContext, hCard);
+	(void)RFClearReaderEventState(rContext, hCard);
 
 	/*
-	 * Allow the status thread to convey information
+	 * Propagate new state to Shared Memory
 	 */
-	if (do_sleep)
-		SYS_USleep(PCSCLITE_STATUS_POLL_RATE + 10);
+	rContext->readerState->readerSharing = rContext->dwContexts;
+	(void)StatSynchronize(rContext->readerState);
 
 	return SCARD_S_SUCCESS;
 }
@@ -834,7 +862,7 @@ LONG SCardDisconnect(SCARDHANDLE hCard, DWORD dwDisposition)
 	{
 		Log1(PCSC_LOG_INFO, "Waiting for release of lock");
 		while (rContext->dwLockId != 0)
-			SYS_USleep(PCSCLITE_LOCK_POLL_RATE);
+			(void)SYS_USleep(PCSCLITE_LOCK_POLL_RATE);
 		Log1(PCSC_LOG_INFO, "Lock released");
 	}
 
@@ -845,7 +873,7 @@ LONG SCardDisconnect(SCARDHANDLE hCard, DWORD dwDisposition)
 	/*
 	 * Unlock any blocks on this context
 	 */
-	rv = RFUnlockSharing(hCard);
+	rv = RFUnlockAllSharing(hCard);
 	if (rv != SCARD_S_SUCCESS)
 		return rv;
 
@@ -879,7 +907,7 @@ LONG SCardDisconnect(SCARDHANDLE hCard, DWORD dwDisposition)
 		/*
 		 * Notify the card has been reset
 		 */
-		RFSetReaderEventState(rContext, SCARD_RESET);
+		(void)RFSetReaderEventState(rContext, SCARD_RESET);
 
 		/*
 		 * Set up the status bit masks on dwStatus
@@ -913,11 +941,6 @@ LONG SCardDisconnect(SCARDHANDLE hCard, DWORD dwDisposition)
 			Log1(PCSC_LOG_DEBUG, "Reset complete.");
 		else
 			Log1(PCSC_LOG_ERROR, "Error resetting card.");
-
-		/*
-		 * Allow the status thread to convey information
-		 */
-		SYS_USleep(PCSCLITE_STATUS_POLL_RATE + 10);
 	}
 	else if (dwDisposition == SCARD_EJECT_CARD)
 	{
@@ -963,25 +986,30 @@ LONG SCardDisconnect(SCARDHANDLE hCard, DWORD dwDisposition)
 	/*
 	 * Remove and destroy this handle
 	 */
-	RFRemoveReaderHandle(rContext, hCard);
-	RFDestroyReaderHandle(hCard);
+	(void)RFRemoveReaderHandle(rContext, hCard);
+	(void)RFDestroyReaderHandle(hCard);
 
 	/*
 	 * For exclusive connection reset it to no connections
 	 */
 	if (rContext->dwContexts == SCARD_EXCLUSIVE_CONTEXT)
-	{
 		rContext->dwContexts = SCARD_NO_CONTEXT;
-		return SCARD_S_SUCCESS;
+	else
+	{
+		/*
+		 * Remove a connection from the context stack
+		 */
+		rContext->dwContexts -= 1;
+
+		if (rContext->dwContexts < 0)
+			rContext->dwContexts = 0;
 	}
 
 	/*
-	 * Remove a connection from the context stack
+	 * Propagate new state to Shared Memory
 	 */
-	rContext->dwContexts -= 1;
-
-	if (rContext->dwContexts < 0)
-		rContext->dwContexts = 0;
+	rContext->readerState->readerSharing = rContext->dwContexts;
+	(void)StatSynchronize(rContext->readerState);
 
 	return SCARD_S_SUCCESS;
 }
@@ -1024,7 +1052,7 @@ LONG SCardBeginTransaction(SCARDHANDLE hCard)
 	/* if the transaction is not yet ready we sleep a bit so the client
 	 * do not retry immediately */
 	if (SCARD_E_SHARING_VIOLATION == rv)
-		SYS_USleep(PCSCLITE_LOCK_POLL_RATE);
+		(void)SYS_USleep(PCSCLITE_LOCK_POLL_RATE);
 
 	Log2(PCSC_LOG_DEBUG, "Status: 0x%08X", rv);
 
@@ -1095,7 +1123,7 @@ LONG SCardEndTransaction(SCARDHANDLE hCard, DWORD dwDisposition)
 		/*
 		 * Notify the card has been reset
 		 */
-		RFSetReaderEventState(rContext, SCARD_RESET);
+		(void)RFSetReaderEventState(rContext, SCARD_RESET);
 
 		/*
 		 * Set up the status bit masks on dwStatus
@@ -1175,7 +1203,7 @@ LONG SCardEndTransaction(SCARDHANDLE hCard, DWORD dwDisposition)
 	/*
 	 * Unlock any blocks on this context
 	 */
-	RFUnlockSharing(hCard);
+	(void)RFUnlockSharing(hCard);
 
 	Log2(PCSC_LOG_DEBUG, "Status: 0x%08X", rv);
 
@@ -1333,12 +1361,18 @@ LONG SCardStatus(SCARDHANDLE hCard, LPSTR mszReaderNames,
 	return rv;
 }
 
-LONG SCardGetStatusChange(SCARDCONTEXT hContext, DWORD dwTimeout,
-	LPSCARD_READERSTATE_A rgReaderStates, DWORD cReaders)
+LONG SCardGetStatusChange(/*@unused@*/ SCARDCONTEXT hContext,
+	/*@unused@*/ DWORD dwTimeout,
+	/*@unused@*/ LPSCARD_READERSTATE_A rgReaderStates,
+	/*@unused@*/ DWORD cReaders)
 {
 	/*
 	 * Client side function
 	 */
+	(void)hContext;
+	(void)dwTimeout;
+	(void)rgReaderStates;
+	(void)cReaders;
 	return SCARD_S_SUCCESS;
 }
 
@@ -1606,6 +1640,9 @@ LONG SCardTransmit(SCARDHANDLE hCard, LPCSCARD_IO_REQUEST pioSendPci,
 
 	sSendPci.Length = pioSendPci->cbPciLength;
 
+	sRecvPci.Protocol = pioRecvPci->dwProtocol;
+	sRecvPci.Length = pioRecvPci->cbPciLength;
+
 	/* the protocol number is decoded a few lines above */
 	Log2(PCSC_LOG_DEBUG, "Send Protocol: T=%d", sSendPci.Protocol);
 
@@ -1621,11 +1658,8 @@ LONG SCardTransmit(SCARDHANDLE hCard, LPCSCARD_IO_REQUEST pioSendPci,
 			cbSendLength, pbRecvBuffer, &dwRxLength, &sRecvPci);
 	}
 
-	if (pioRecvPci)
-	{
-		pioRecvPci->dwProtocol = sRecvPci.Protocol;
-		pioRecvPci->cbPciLength = sRecvPci.Length;
-	}
+	pioRecvPci->dwProtocol = sRecvPci.Protocol;
+	pioRecvPci->cbPciLength = sRecvPci.Length;
 
 	/*
 	 * Check for any errors that might have occurred
@@ -1654,20 +1688,27 @@ LONG SCardTransmit(SCARDHANDLE hCard, LPCSCARD_IO_REQUEST pioSendPci,
 	return SCARD_S_SUCCESS;
 }
 
-LONG SCardListReaders(SCARDCONTEXT hContext, LPCSTR mszGroups,
-	LPSTR mszReaders, LPDWORD pcchReaders)
+LONG SCardListReaders(/*@unused@*/ SCARDCONTEXT hContext,
+	/*@unused@*/ LPCSTR mszGroups,
+	/*@unused@*/ LPSTR mszReaders,
+	/*@unused@*/ LPDWORD pcchReaders)
 {
 	/*
 	 * Client side function
 	 */
+	(void)hContext;
+	(void)mszGroups;
+	(void)mszReaders;
+	(void)pcchReaders;
 	return SCARD_S_SUCCESS;
 }
 
-LONG SCardCancel(SCARDCONTEXT hContext)
+LONG SCardCancel(/*@unused@*/ SCARDCONTEXT hContext)
 {
 	/*
 	 * Client side function
 	 */
+	(void)hContext;
 	return SCARD_S_SUCCESS;
 }
 
