@@ -6,7 +6,7 @@
  *  Damien Sauveron <damien.sauveron@labri.fr>
  *  Ludoic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: winscard_msg.c 2961 2008-05-16 09:12:53Z rousseau $
+ * $Id: winscard_msg.c 3250 2009-01-02 13:32:17Z rousseau $
  */
 
 /**
@@ -96,7 +96,7 @@ INTERNAL int SHMClientSetupSession(uint32_t *pdwClientID)
 	{
 		Log3(PCSC_LOG_CRITICAL, "Error: connect to client socket %s: %s",
 			PCSCLITE_CSOCK_NAME, strerror(errno));
-		SYS_CloseFile(*pdwClientID);
+		(void)SYS_CloseFile(*pdwClientID);
 		return -1;
 	}
 
@@ -105,7 +105,7 @@ INTERNAL int SHMClientSetupSession(uint32_t *pdwClientID)
 	{
 		Log3(PCSC_LOG_CRITICAL, "Error: cannot set socket %s nonblocking: %s",
 			PCSCLITE_CSOCK_NAME, strerror(errno));
-		SYS_CloseFile(*pdwClientID);
+		(void)SYS_CloseFile(*pdwClientID);
 		return -1;
 	}
 
@@ -121,7 +121,7 @@ INTERNAL int SHMClientSetupSession(uint32_t *pdwClientID)
  */
 INTERNAL int SHMClientCloseSession(uint32_t dwClientID)
 {
-	SYS_CloseFile(dwClientID);
+	(void)SYS_CloseFile(dwClientID);
 	return 0;
 }
 
@@ -140,7 +140,7 @@ INTERNAL int SHMClientCloseSession(uint32_t dwClientID)
  * @retval -1 Socket is closed.
  * @retval -1 A signal was received.
  */
-INTERNAL int SHMMessageSend(void *buffer_void, uint64_t buffer_size,
+INTERNAL int32_t SHMMessageSend(void *buffer_void, uint64_t buffer_size,
 	int32_t filedes, int32_t blockAmount)
 {
 	char *buffer = buffer_void;
@@ -265,7 +265,7 @@ INTERNAL int SHMMessageSend(void *buffer_void, uint64_t buffer_size,
  * @retval -1 Socket is closed.
  * @retval -1 A signal was received.
  */
-INTERNAL int SHMMessageReceive(void *buffer_void, uint64_t buffer_size,
+INTERNAL int32_t SHMMessageReceive(void *buffer_void, uint64_t buffer_size,
 	int32_t filedes, int32_t blockAmount)
 {
 	char *buffer = buffer_void;
@@ -477,7 +477,7 @@ INTERNAL int32_t WrapSHMWrite(uint32_t command, uint32_t dwClientID,
  */
 INTERNAL void SHMCleanupSharedSegment(int sockValue, const char *pcFilePath)
 {
-	SYS_CloseFile(sockValue);
-	SYS_Unlink(pcFilePath);
+	(void)SYS_CloseFile(sockValue);
+	(void)SYS_RemoveFile(pcFilePath);
 }
 
