@@ -6,7 +6,7 @@
  *  Damien Sauveron <damien.sauveron@labri.fr>
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: winscard_svc.c 3031 2008-06-27 07:31:37Z rousseau $
+ * $Id: winscard_svc.c 3267 2009-01-05 13:58:14Z rousseau $
  */
 
 /**
@@ -23,6 +23,7 @@
 #include <time.h>
 #include <stdio.h>
 #include <string.h>
+#include <stddef.h>
 
 #include "pcscd.h"
 #include "winscard.h"
@@ -48,13 +49,13 @@ static struct _psContext
 	int protocol_major, protocol_minor;	/**< Protocol number agreed between client and server*/
 } psContext[PCSCLITE_MAX_APPLICATIONS_CONTEXTS];
 
-LONG MSGCheckHandleAssociation(SCARDHANDLE, DWORD);
-LONG MSGFunctionDemarshall(psharedSegmentMsg, DWORD);
-LONG MSGAddContext(SCARDCONTEXT, DWORD);
-LONG MSGRemoveContext(SCARDCONTEXT, DWORD);
-LONG MSGAddHandle(SCARDCONTEXT, SCARDHANDLE, DWORD);
-LONG MSGRemoveHandle(SCARDHANDLE, DWORD);
-LONG MSGCleanupClient(DWORD);
+static LONG MSGCheckHandleAssociation(SCARDHANDLE, DWORD);
+static LONG MSGFunctionDemarshall(psharedSegmentMsg, DWORD);
+static LONG MSGAddContext(SCARDCONTEXT, DWORD);
+static LONG MSGRemoveContext(SCARDCONTEXT, DWORD);
+static LONG MSGAddHandle(SCARDCONTEXT, SCARDHANDLE, DWORD);
+static LONG MSGRemoveHandle(SCARDHANDLE, DWORD);
+static LONG MSGCleanupClient(DWORD);
 
 static void ContextThread(LPVOID pdwIndex);
 
@@ -100,7 +101,7 @@ LONG CreateContextThread(uint32_t *pdwClientID)
 		(PCSCLITE_THREAD_FUNCTION( )) ContextThread, (LPVOID) i);
 	if (rv)
 	{
-		SYS_CloseFile(psContext[i].dwClientID);
+		(void)SYS_CloseFile(psContext[i].dwClientID);
 		psContext[i].dwClientID = 0;
 		Log2(PCSC_LOG_CRITICAL, "SYS_ThreadCreate failed: %s", strerror(rv));
 		return SCARD_E_NO_MEMORY;
@@ -133,7 +134,7 @@ static void ContextThread(LPVOID dwIndex)
 
 	while (1)
 	{
-		switch (rv = SHMProcessEventsContext(psContext[dwContextIndex].dwClientID, &msgStruct, 0))
+		switch (rv = SHMProcessEventsContext(psContext[dwContextIndex].dwClientID, &msgStruct))
 		{
 		case 0:
 			if (msgStruct.mtype == CMD_CLIENT_DIED)
@@ -143,8 +144,8 @@ static void ContextThread(LPVOID dwIndex)
 				 */
 				Log2(PCSC_LOG_DEBUG, "Client die: %d",
 					psContext[dwContextIndex].dwClientID);
-				MSGCleanupClient(dwContextIndex);
-				SYS_ThreadExit((LPVOID) NULL);
+				(void)MSGCleanupClient(dwContextIndex);
+				(void)SYS_ThreadExit((LPVOID) NULL);
 			}
 			break;
 
@@ -159,9 +160,9 @@ static void ContextThread(LPVOID dwIndex)
 				{
 					Log2(PCSC_LOG_DEBUG, "MSGFunctionDemarshall failed: %d",
 						rv);
-					SHMClientCloseSession(psContext[dwContextIndex].dwClientID);
-					MSGCleanupClient(dwContextIndex);
-					SYS_ThreadExit((LPVOID) NULL);
+					(void)SHMClientCloseSession(psContext[dwContextIndex].dwClientID);
+					(void)MSGCleanupClient(dwContextIndex);
+					(void)SYS_ThreadExit((LPVOID) NULL);
 				}
 
 				/* the SCARD_TRANSMIT_EXTENDED anwser is already sent by
@@ -252,7 +253,8 @@ static void ContextThread(LPVOID dwIndex)
  * @param[in] msgStruct Message to be demarshalled and executed.
  * @param[in] dwContextIndex
  */
-LONG MSGFunctionDemarshall(psharedSegmentMsg msgStruct, DWORD dwContextIndex)
+static LONG MSGFunctionDemarshall(psharedSegmentMsg msgStruct,
+	DWORD dwContextIndex)
 {
 	LONG rv;
 	establish_struct *esStr;
@@ -472,11 +474,11 @@ LONG MSGFunctionDemarshall(psharedSegmentMsg msgStruct, DWORD dwContextIndex)
 			{
 				/* copy the first APDU part */
 				memcpy(pbSendBuffer, treStr->data,
-					PCSCLITE_MAX_MESSAGE_SIZE-sizeof(*treStr));
+					PCSCLITE_MAX_MESSAGE_SIZE-offsetof(transmit_struct_extended, data));
 
 				/* receive the second block */
 				rv = SHMMessageReceive(
-					pbSendBuffer+PCSCLITE_MAX_MESSAGE_SIZE-sizeof(*treStr),
+					pbSendBuffer+PCSCLITE_MAX_MESSAGE_SIZE-offsetof(transmit_struct_extended, data),
 					treStr->size - PCSCLITE_MAX_MESSAGE_SIZE,
 					psContext[dwContextIndex].dwClientID,
 					PCSCLITE_SERVER_ATTEMPTS);
@@ -503,12 +505,12 @@ LONG MSGFunctionDemarshall(psharedSegmentMsg msgStruct, DWORD dwContextIndex)
 			treStr->pioRecvPciLength = ioRecvPci.cbPciLength;
 			treStr->pcbRecvLength = cbRecvLength;
 
-			treStr->size = sizeof(*treStr) + treStr->pcbRecvLength;
+			treStr->size = offsetof(transmit_struct_extended, data) + treStr->pcbRecvLength;
 			if (treStr->size > PCSCLITE_MAX_MESSAGE_SIZE)
 			{
 				/* two blocks */
 				memcpy(treStr->data, pbRecvBuffer, PCSCLITE_MAX_MESSAGE_SIZE
-					- sizeof(*treStr));
+					- offsetof(transmit_struct_extended, data));
 
 				rv = SHMMessageSend(msgStruct, sizeof(*msgStruct),
 					psContext[dwContextIndex].dwClientID,
@@ -517,7 +519,7 @@ LONG MSGFunctionDemarshall(psharedSegmentMsg msgStruct, DWORD dwContextIndex)
 					Log1(PCSC_LOG_CRITICAL, "transmission failed");
 
 				rv = SHMMessageSend(pbRecvBuffer + PCSCLITE_MAX_MESSAGE_SIZE
-					- sizeof(*treStr),
+					- offsetof(transmit_struct_extended, data),
 					treStr->size - PCSCLITE_MAX_MESSAGE_SIZE,
 					psContext[dwContextIndex].dwClientID,
 					PCSCLITE_SERVER_ATTEMPTS);
@@ -619,13 +621,13 @@ LONG MSGFunctionDemarshall(psharedSegmentMsg msgStruct, DWORD dwContextIndex)
 	return 0;
 }
 
-LONG MSGAddContext(SCARDCONTEXT hContext, DWORD dwContextIndex)
+static LONG MSGAddContext(SCARDCONTEXT hContext, DWORD dwContextIndex)
 {
 	psContext[dwContextIndex].hContext = hContext;
 	return SCARD_S_SUCCESS;
 }
 
-LONG MSGRemoveContext(SCARDCONTEXT hContext, DWORD dwContextIndex)
+static LONG MSGRemoveContext(SCARDCONTEXT hContext, DWORD dwContextIndex)
 {
 	int i;
 	LONG rv;
@@ -674,10 +676,10 @@ LONG MSGRemoveContext(SCARDCONTEXT hContext, DWORD dwContextIndex)
 				}
 
 				if (rv == SCARD_W_RESET_CARD || rv == SCARD_W_REMOVED_CARD)
-					SCardDisconnect(psContext[dwContextIndex].hCard[i],
+					(void)SCardDisconnect(psContext[dwContextIndex].hCard[i],
 						SCARD_LEAVE_CARD);
 				else
-					SCardDisconnect(psContext[dwContextIndex].hCard[i],
+					(void)SCardDisconnect(psContext[dwContextIndex].hCard[i],
 						SCARD_RESET_CARD);
 
 				psContext[dwContextIndex].hCard[i] = 0;
@@ -691,7 +693,8 @@ LONG MSGRemoveContext(SCARDCONTEXT hContext, DWORD dwContextIndex)
 	return SCARD_E_INVALID_VALUE;
 }
 
-LONG MSGAddHandle(SCARDCONTEXT hContext, SCARDHANDLE hCard, DWORD dwContextIndex)
+static LONG MSGAddHandle(SCARDCONTEXT hContext, SCARDHANDLE hCard,
+	DWORD dwContextIndex)
 {
 	int i;
 
@@ -723,7 +726,7 @@ LONG MSGAddHandle(SCARDCONTEXT hContext, SCARDHANDLE hCard, DWORD dwContextIndex
 	return SCARD_E_INVALID_VALUE;
 }
 
-LONG MSGRemoveHandle(SCARDHANDLE hCard, DWORD dwContextIndex)
+static LONG MSGRemoveHandle(SCARDHANDLE hCard, DWORD dwContextIndex)
 {
 	int i;
 
@@ -740,7 +743,7 @@ LONG MSGRemoveHandle(SCARDHANDLE hCard, DWORD dwContextIndex)
 }
 
 
-LONG MSGCheckHandleAssociation(SCARDHANDLE hCard, DWORD dwContextIndex)
+static LONG MSGCheckHandleAssociation(SCARDHANDLE hCard, DWORD dwContextIndex)
 {
 	int i;
 
@@ -754,17 +757,18 @@ LONG MSGCheckHandleAssociation(SCARDHANDLE hCard, DWORD dwContextIndex)
 
 	/* Must be a rogue client, debug log and sleep a couple of seconds */
 	Log1(PCSC_LOG_ERROR, "Client failed to authenticate");
-	SYS_Sleep(2);
+	(void)SYS_Sleep(2);
 
 	return -1;
 }
 
-LONG MSGCleanupClient(DWORD dwContextIndex)
+static LONG MSGCleanupClient(DWORD dwContextIndex)
 {
 	if (psContext[dwContextIndex].hContext != 0)
 	{
-		SCardReleaseContext(psContext[dwContextIndex].hContext);
-		MSGRemoveContext(psContext[dwContextIndex].hContext, dwContextIndex);
+		(void)SCardReleaseContext(psContext[dwContextIndex].hContext);
+		(void)MSGRemoveContext(psContext[dwContextIndex].hContext,
+			dwContextIndex);
 	}
 
 	psContext[dwContextIndex].dwClientID = 0;
